require "vector"
require "std"

function d20()
  local object={}

  -- every object needs a mass, angular mass, position, angular velocity, etc, etc.
  object.position=Vector{0,0,0}
  object.mass=1
  object.velocity=Vector{0,0,0}
  object.angularMass=4
  object.angularVelocity=Vector{0,0,0}
  object.color=0xFFFF00
  object.radius=1
  
  -- we take a d8 and split every vertex into two
  object.vertices={
    {1,0,0},{0,1,0},{-1,0,0},{0,-1,0},
    {0,0,1},{0,0,-1}
  }
  for i=1,#object.vertices do
    local v=object.vertices[i]
    local split=Vector{v[1],v[2],v[3]}
    for c=1, 3 do if v[c]~=0 then --fint the nonzero coordinate and modify the next
      c=math.mod1(c+1,3)
      v[c]= 1/1.6180339887498948482
      split[c]=-v[c]
      table.insert(object.vertices,split)
    break end end
  
  end  
  table.transform(object.vertices, Vector)
  
  --we also rescale because it's too big
  table.transform(object.vertices, function(v) v[1],v[2],v[3]=unpack(v*0.8) return v end)
  
  
  -- every object has a set of surfaces that hold the polygons and the textures
  local polygons=verticesToPolygons(object.vertices)
  object.surfaces=table.transform(polygons,polygonToSurface)
  
  --every surface also has a texture that contains the number image.
  for i,surface in ipairs(object.surfaces) do
    surface.texture={
      image=i..".png",
      o=surface.center,
      y=math.avg(surface.polygon[3], surface.polygon[2], surface.center)
    }
    surface.texture.x=vector.cross(surface.normal, (surface.texture.y-surface.texture.o))+surface.texture.o
  end
  
  -- every object also has a set of vectors that need to be updated with rotation.
  object.allVertices=extractAllVertices(object)

  -- also surfaces refer back to their parent object
  for i,surface in ipairs(object.surfaces) do 
    surface.object=object
  end
  
return object end
